# Gitlab CI/CD Testing Repo

This repo is used for a situation where students have their own Gitlab repo and tests are provided by their lecturer.
The "tests" folder contains the tests for the examples, all other files are for the Gitlab CI/CD.

## I'm a student, what do I need to know?

### Which test types exist? 

If your repo includes this pipeline, you need to know the following things in order to utilize the provided tests and structure:

There are two types of tests: Xunit and shell tests.

Xunit is a testing framework for the .Net ecosystem, in which classes contain functions that evaluate elements of your code.

Shell tests evaluate the output of your main function. For example, the lecturer can test for a specific output format, 
such as 'Student X: XXX€', and the test will fail if any other string is output.

Success:  
Student 3: 321€  
Student 6: 98€  
Student 98: 999€

Error:  
Stud 1: 132€  
Student: 1 12€  
Student 1: 12 Euro  
@@@@@@@@@  
Student 98:

### Where can I find the tests?

All tests are present in the "tests" subfolder of this repo.

### How is this all working?

A Gitlab pipeline is executed everytime you push code to your repo. You can view the last executed pipeline by clicking the icon-button on the left side of the commit hash
at the overview of your repo. Now you can see the pipeline. If you click on the test job, you can view the output of the testing script. Green and blue lines mark the start and end of a single test. Inside those marks is the test output, which tells you what you did wrong or right. If you go back and open the "Tests" tab, you can see a entry called "merge-job", click on it. Now you see every test that was executed. Green tests were successful and red tests had an error. You can get more information by clicking on the "View details" button. Most of the time there is a path were you can find the test file, in order to understand what you did wrong. For finding the corresponding test, only consider the path after the name of the test repo.

## I'm a lecturer, what do I need to know?

You have two choices for tests: Xunit or write your own shell test in which you can access the output of the students program.

### Xunit

Develop your Xunit test seperately and then copy the project files to the corresponding folder. You need to pay attention to these criteria:
- no reference to the students project, gets added automatically
- Dotnet 6
- "Syntax errors" in the repo are normal because there is no reference to the students code 

### Shell

Follow these steps to create a shell test:
- add a tests.sh bash script to a folder inside tests/
- tests.sh will get executed by the pipeline
  - tests.sh gets as $1 the student project file and is in charge of running it as it likes. $2 is the desired location for the program output
    Execute it with `dotnet run --project "$1" > "$2"`
- (If needed) split up single tests into their own bash script:
  - enable execute rights for the sub-script
  - execute the sub-script inside the tests.sh script
  - parse the $1 variable to your sub-script as an argument
- analyze the $1 variable however you want (regex, e.g.)
- finally, save the testcase report to the "TestResults.txt" file like so
  - Success: `echo "Test|TestClass" >> TestResults.txt`
  - Error: `echo "Test|TestClass|Failure Description" >> TestResults.txt`
  - You can append multiple testcases to the final report with this syntax
- Exit with another code than 0 if the test should be marked as "failed"

### Both

- add the projects path to /tests/paths.txt
