#!/bin/bash

function convertTxtResultsToXml {
    currDir=$(pwd)
    reportsFolder="${currDir}/TestResults"

    if ! [[ -d "${reportsFolder}" ]] ; then
        mkdir "TestResults"
    fi

    currentDate=$(date +%Y-%m-%dT%H:%M:%S)

    allCases=""
    nrCases=0
    nrFailures=0

    if ! [[ -f "TestResults.txt" ]]; then
        return 1
    fi

    while read -r line || [ -n "$line" ]
    do
        ((nrCases++))
        IFS='|'; splitLine=($line); unset IFS;

        name=${splitLine[0]}
        className=${splitLine[1]}
        if ! [ -z "${splitLine[2]}" ]; then
            failedDescr=${splitLine[2]}
            allCases+=$(create_testcase "$name" "$className" "$failedDescr")
            ((nrFailures++))
        else
            allCases+=$(create_testcase "$name" "$className")
        fi
    done < TestResults.txt

    REPORT=$(cat <<EOF
        <?xml version="1.0" encoding="utf-8"?>
        <testsuites>
            <testsuite tests="${nrCases}" failures="${nrFailures}" skipped="0" errors="0" timestamp="${currentDate}">
                ${allCases}
            </testsuite>
        </testsuites>
EOF
)

    echo $REPORT > "${reportsFolder}/JunitTestResults.xml"
    echo "Converted shell tests to junit report"
}

function create_testcase {
    # $1=name, $2=className, $3=failureText
    if ! [ -z ${3+x} ]; then
        # failure
        FAILURE="<failure type='' message=''><![CDATA[$3]]></failure>"
    else
        # no failure
        FAILURE=""
    fi
    REPORT=$(cat <<EOF
        <testcase name="${1}" classname="${2}">
            ${FAILURE}
        </testcase>
EOF
    )
    echo "${REPORT}"
}

path=${1}

cd tests

if ! [[ -d $path ]] ; then
    echo "Could not find folder to convert reports" 1>&2
    exit 1
fi

cd $path
if [[ -f "TestResults.txt" ]]; then
    convertTxtResultsToXml
elif [[ -f "TestResults/TestResults.xml" ]]; then
    dotnet xunit-to-junit TestResults/TestResults.xml TestResults/JunitTestResults.xml
fi

exit 0
