#!/bin/bash

OUTPUT_FILE='.gitlab-ci.yml'

if [[ -f $OUTPUT_FILE ]] ; then
  rm $OUTPUT_FILE
fi
touch $OUTPUT_FILE
cat ./pipeline/header.yml >> $OUTPUT_FILE
cat ./pipeline/jobs.yml >> $OUTPUT_FILE

ROOT_FOLDER=$(pwd)
allTests=()

while read -r path || [ -n "$path" ]; do
  clean_path=$(echo "${path//[$'\t\r\n ']}")
  studentRepo=../$path
  testRepo=tests/$path
  testName=Test-$clean_path
  allTests+=($testName)

  echo "" >> $OUTPUT_FILE

  jobTemplate=$(cat << EOF
$testName:
  stage: test
  dependencies:
    - clone
  allow_failure: true
  script:
    - cd \$PROJECT_NAME
    - ./testProject.sh "$clean_path"
  after_script:
    - cd \$PROJECT_NAME
    - ./convertReport.sh "$clean_path"
  artifacts:
    when: always
    paths:
      - \$PROJECT_NAME/tests/$clean_path/TestResults/
    expire_in: 10 mins
  rules:
    - changes:
      - $clean_path/**/*
EOF
)
  echo "$jobTemplate" >> $OUTPUT_FILE

done < tests/paths.txt

jobString+="\n    - \"clone\""
jobString+="\n    - \"Test-disallowed-folders\""
for job in ${allTests[@]}; do
  jobString+="\n    - \"$job\""
done

mergeTemplate=$(cat << EOF
merge:
  stage: merge
  dependencies:$(printf "$jobString")
  image: python:latest
  script:
    - cd \$PROJECT_NAME && ls -la
    - ./mergeReports.sh
  artifacts:
    paths:
      - \$PROJECT_NAME/mergedReports.xml
    reports:
      junit: \$PROJECT_NAME/mergedReports.xml
EOF
)

echo -e "$mergeTemplate" >> $OUTPUT_FILE
