#!/bin/bash

SCRIPT_FOLDER=$(pwd)

BLUE='\033[0;34m'
GREEN='\033[0;32m'
RED='\033[0;31m'
NC='\033[0m' # No Color


cd tests

path=${1}
caseFailed=0

if ! [[ -d $path ]] ; then
    echo "Could not find folder to test in" 1>&2
    exit 1
fi

cd $path

studentProject="$SCRIPT_FOLDER/../$path/*.csproj"

# check if students Project even exists
if ! compgen -G "$studentProject" > /dev/null; then
    echo "Students project $path not found"
    exit 0
fi


if [[ -f "tests.sh" ]] ; then
    echo -e "${BLUE}@@@@@@@@@@@@@@@@@@@@@@@@@ TEST START @@@@@@@@@@@@@@@@@@@@@@@@@@@${NC}"
    echo "Running shell test ..."
    programOutputFile=$( mktemp )
    studentProjectAbsolute=$( readlink -f $studentProject )
    bash ./tests.sh "$studentProjectAbsolute" "$programOutputFile"
    EXIT_CODE=$?

    echo "Recieved following Output (exit code $EXIT_CODE):"
    cat $programOutputFile

    if [[ $EXIT_CODE != 0 ]]; then
        caseFailed=1
    fi
    echo -e "${BLUE}@@@@@@@@@@@@@@@@@@@@@@@@@ TEST END @@@@@@@@@@@@@@@@@@@@@@@@@@@${NC}"
else
    dotnet new tool-manifest
    dotnet tool install --local dotnet-xunit-to-junit
    dotnet add package XunitXml.TestLogger
    dotnet add reference $studentProject
    dotnet restore

    echo -e "${BLUE}@@@@@@@@@@@@@@@@@@@@@@@@@ TEST START @@@@@@@@@@@@@@@@@@@@@@@@@@@${NC}"
    echo "Running xunit test ..."
    dotnet test --blame --logger:"xunit;MethodFormat=Class;FailureBodyFormat=Verbose"
    EXIT_CODE=$?
    echo -e "${BLUE}@@@@@@@@@@@@@@@@@@@@@@@@@ TEST END @@@@@@@@@@@@@@@@@@@@@@@@@@@${NC}"
    if [[ $EXIT_CODE != 0 ]]; then
        caseFailed=1
    fi
fi

exit $caseFailed
